// 
// Copyright (C) 2019, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//
#region Using declarations
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using SharpDX;
using SharpDX.Direct2D1;
using System;

//added
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Xml.Serialization;
using NinjaTrader.Gui;
#endregion

namespace NinjaTrader.NinjaScript.ChartStyles
{
	public class MultiPeriodCandles : ChartStyle
	{
		private object icon;

		public override int GetBarPaintWidth(int barWidth)
		{
			return 10 - 2 * (int) Math.Round(Stroke.Width);
		}

		public override object Icon
		{
			get { return icon ?? (icon = NinjaTrader.Gui.Tools.Icons.ChartBox2); }
		}

		public override void OnRender(ChartControl chartControl, ChartScale chartScale, ChartBars chartBars)
		{
			Bars			bars				= chartBars.Bars;
			float			chartMinX			= ConvertToHorizontalPixels(chartControl, chartControl.CanvasLeft + chartControl.Properties.BarMarginRight);
			RectangleF		rect				= new RectangleF();
			Vector2			point0			= new Vector2();
			Vector2			point1			= new Vector2();
			int				toIndex				= chartBars.ToIndex;
			
			SharpDX.Direct2D1.StrokeStyleProperties ssPropsWick = new SharpDX.Direct2D1.StrokeStyleProperties();
			ssPropsWick.DashStyle								= DashStyleWick;
			SharpDX.Direct2D1.StrokeStyle styleWick 			= new SharpDX.Direct2D1.StrokeStyle(RenderTarget.Factory, ssPropsWick);
			SharpDX.Direct2D1.AntialiasMode oldAntialiasMode	= RenderTarget.AntialiasMode;
			RenderTarget.AntialiasMode							= SharpDX.Direct2D1.AntialiasMode.Aliased;

			if (toIndex >= 0 && toIndex < bars.Count - 1)
				toIndex++;

			for (int idx = chartBars.FromIndex; idx <= toIndex; idx++)
			{
				double	closeValue				= bars.GetClose(idx);
				float	high					= chartScale.GetYByValue(bars.GetHigh(idx));
				float	low						= chartScale.GetYByValue(bars.GetLow(idx));
				float	open					= chartScale.GetYByValue(bars.GetOpen(idx));
				float	close					= chartScale.GetYByValue(bars.GetClose(idx));
				double	openValue				= bars.GetOpen(idx);
				Brush	overriddenBarBrush		= chartControl.GetBarOverrideBrush(chartBars, idx);
				Brush	overriddenOutlineBrush	= chartControl.GetCandleOutlineOverrideBrush(chartBars, idx);
				float	x						= chartControl.GetXByBarIndex(chartBars, idx);
				float	boxStartPosition;
				
				Brush		upOutBrushDX			= UpOutBrush.ToDxBrush(RenderTarget);
				Brush		dnOutBrushDX			= DnOutBrush.ToDxBrush(RenderTarget);
				Brush		dojiBrushDX				= DojiBrush.ToDxBrush(RenderTarget);
				
				Brush brush	= closeValue >= openValue ? UpBrushDX : DownBrushDX;
				brush.Opacity	= Opacity * 0.01f;
				
				if (idx == chartBars.FromIndex && (toIndex == 0 || idx == 0))
				{
					if (toIndex == 0)
						boxStartPosition = chartMinX;
					else
						boxStartPosition = 2 * x - chartControl.GetXByBarIndex(chartBars, idx + 1);
				}
				else
					boxStartPosition = chartControl.GetXByBarIndex(chartBars, idx - 1);
				
				float midpoint = boxStartPosition + (x - boxStartPosition) / 2;

				if (Math.Abs(x - boxStartPosition) < 0.2)
					continue;

				float width = Math.Max(2f, Math.Abs(x - boxStartPosition));

				if (closeValue > openValue)
				{
					width		-= Stroke.Width;
					rect.X		= boxStartPosition;
					rect.Y		= close;
					rect.Width	= width;
					rect.Height	= open - close;
					TransformBrush(overriddenBarBrush ?? UpBrushDX, rect);
					TransformBrush(overriddenOutlineBrush ?? upOutBrushDX, rect);
					RenderTarget.FillRectangle(rect, overriddenBarBrush ?? brush);
					RenderTarget.DrawRectangle(rect, overriddenOutlineBrush ?? upOutBrushDX, WidthOutline, Stroke.StrokeStyle);
					
					if(BoxHighLow)
					{
						rect.X		= boxStartPosition;
						rect.Y		= high;
						rect.Width	= width;
						rect.Height	= low - high;
						TransformBrush(overriddenOutlineBrush ?? upOutBrushDX, rect);
						RenderTarget.DrawRectangle(rect, overriddenOutlineBrush ?? upOutBrushDX, WidthOutline, Stroke.StrokeStyle);
					}
				}
				else
				{
					width		-= Stroke2.Width;
					rect.X		= boxStartPosition;
					rect.Y		= open;
					rect.Width	= width;
					rect.Height	= close - open;
					TransformBrush(overriddenBarBrush ?? DownBrushDX, rect);
					TransformBrush(overriddenOutlineBrush ?? dnOutBrushDX, rect);
					RenderTarget.FillRectangle(rect, overriddenBarBrush ?? brush);
					RenderTarget.DrawRectangle(rect, overriddenOutlineBrush ?? dnOutBrushDX, WidthOutline, Stroke2.StrokeStyle);
					if(BoxHighLow)
					{
						rect.X		= boxStartPosition;
						rect.Y		= high;
						rect.Width	= width;
						rect.Height	= low - high;
						TransformBrush(overriddenOutlineBrush ?? dnOutBrushDX, rect);
						RenderTarget.DrawRectangle(rect, overriddenOutlineBrush ?? dnOutBrushDX, WidthOutline, Stroke2.StrokeStyle);
					}
				}
				
				// High wick
				if (high < Math.Min(open, close))
				{
					point0.X = midpoint;
					point0.Y = high;
					point1.X = midpoint;
					point1.Y = Math.Min(open, close);
					TransformBrush(overriddenOutlineBrush ?? (closeValue == openValue ? dojiBrushDX : closeValue > openValue ? upOutBrushDX : dnOutBrushDX), new RectangleF(point0.X - WidthWick, point0.Y, WidthWick, point1.Y - point0.Y));
					RenderTarget.DrawLine(point0, point1, overriddenOutlineBrush ?? (closeValue == openValue ? dojiBrushDX : closeValue > openValue ? upOutBrushDX : dnOutBrushDX), WidthWick, styleWick);

				}

				// Low wick
				if (low > Math.Max(open, close))
				{
					point0.X = midpoint;
					point0.Y = low;
					point1.X = midpoint;
					point1.Y = Math.Max(open, close);
					TransformBrush(overriddenOutlineBrush ?? (closeValue == openValue ? dojiBrushDX : closeValue > openValue ? upOutBrushDX : dnOutBrushDX), new RectangleF(point1.X - WidthWick, point1.Y, WidthWick, point0.Y - point1.Y));
					RenderTarget.DrawLine(point0, point1, overriddenOutlineBrush ?? (closeValue == openValue ? dojiBrushDX : closeValue > openValue ? upOutBrushDX : dnOutBrushDX), WidthWick, styleWick);
				}
			}
		}

		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Name			= "MultiPeriodCandles";
				ChartStyleType	= (ChartStyleType) 512;
				BarWidth		= 1;
				
				this.UpBrush		= System.Windows.Media.Brushes.Green;
				this.DownBrush 		= System.Windows.Media.Brushes.DarkRed;
				this.UpOutBrush 	= System.Windows.Media.Brushes.LimeGreen;
				this.DnOutBrush 	= System.Windows.Media.Brushes.Red;
				this.DojiBrush 		= System.Windows.Media.Brushes.Gray;
				WidthOutline		= 1f;
				WidthWick			= 1f;
				Opacity				= 85;
				BoxHighLow			= false;
			}
			else if (State == State.Configure)
			{
				Properties.Remove(Properties.Find("BarWidthUI", true));

				SetPropertyName("DownBrush",	Custom.Resource.NinjaScriptChartStyleBoxDownBarsColor);
				SetPropertyName("UpBrush",		Custom.Resource.NinjaScriptChartStyleBoxUpBarsColor);
				SetPropertyName("Stroke",		Custom.Resource.NinjaScriptChartStyleBoxUpBarsOutline);
				SetPropertyName("Stroke2",		Custom.Resource.NinjaScriptChartStyleBoxDownBarsOutline);
				
				
				SetPropertyName("BarWidth",		Custom.Resource.NinjaScriptChartStyleBarWidth);
				
				Properties.Remove(Properties.Find("Stroke", true));
				Properties.Remove(Properties.Find("Stroke2", true));
				
				UpBrush.Freeze();
				DownBrush.Freeze();
				UpOutBrush.Freeze();
				DnOutBrush.Freeze();
				DojiBrush.Freeze();
			}
		}
		
		#region Properties

		[XmlIgnore]
		[Display(Name="Colors - Down bar outline", Description="Outline color of down bar", Order=1, GroupName="Chart Style")]
		public System.Windows.Media.Brush DnOutBrush
		{ get; set; }

		[Browsable(false)]
		public string DnOutBrushSerializable
		{
			get { return Serialize.BrushToString(DnOutBrush); }
			set { DnOutBrush = Serialize.StringToBrush(value); }
		}
		
		[XmlIgnore]
		[Display(Name="Colors - Up bar outline", Description="Outline color of up bar", Order=2, GroupName="Chart Style")]
		public System.Windows.Media.Brush UpOutBrush
		{ get; set; }

		[Browsable(false)]
		public string UpOutBrushSerializable
		{
			get { return Serialize.BrushToString(UpOutBrush); }
			set { UpOutBrush = Serialize.StringToBrush(value); }
		}
	
		[XmlIgnore]
		[Display(Name="Colors - Doji outline", Description="Color of doji bar", Order=3, GroupName="Chart Style")]
		public System.Windows.Media.Brush DojiBrush
		{ get; set; }

		[Browsable(false)]
		public string DojiBrushSerializable
		{
			get { return Serialize.BrushToString(DojiBrush); }
			set { DojiBrush = Serialize.StringToBrush(value); }
		}
		
		[Range(0, 100),NinjaScriptProperty]
		[Display(Name="Opacity % (0-100)", Description="Opacity of bar.", Order=4, GroupName="Chart Style")]
		public int Opacity
		{ get; set; }
		
		[Range(1.0f, 20.0f),NinjaScriptProperty]
		[Display(Name="Outline width", Description="Outline width.", Order=6, GroupName="Chart Style")]
		public float WidthOutline
		{ get; set; }
		
		[NinjaScriptProperty]
		[Display(Name="Wick dashstyle", Description="Wick dashstyle.", Order=7, GroupName="Chart Style")]
		public SharpDX.Direct2D1.DashStyle DashStyleWick
		{ get; set; }
		
		[Range(1.0f, 20.0f),NinjaScriptProperty]
		[Display(Name="Wick width", Description="Wick width.", Order=8, GroupName="Chart Style")]
		public float WidthWick
		{ get; set; }
		
		[NinjaScriptProperty]
		public bool BoxHighLow
		{ get; set; }
		
		#endregion
	}
}
